package java.security;

import sun.security.util.DerValue;
import sun.security.util.Debug;
import sun.security.util.DerInputStream;
import java.math.BigInteger;
import sun.security.util.DerOutputStream;
import java.util.Arrays;
import java.io.IOException;
import sun.security.util.ObjectIdentifier;
import java.util.regex.Pattern;

public final class PKCS12Attribute implements KeyStore.Entry.Attribute
{
    private static final Pattern COLON_SEPARATED_HEX_PAIRS;
    private String name;
    private String value;
    private byte[] encoded;
    private int hashValue;
    
    public PKCS12Attribute(String s, String value) {
        this.hashValue = -1;
        if (s == null || value == null) {
            throw new NullPointerException();
        }
        ObjectIdentifier objectIdentifier;
        try {
            objectIdentifier = new ObjectIdentifier(s);
        }
        catch (IOException cause) {
            throw new IllegalArgumentException("Incorrect format: name", cause);
        }
        this.name = s;
        int length = value.length();
        String[] split;
        if (value.charAt(0) == '[' && value.charAt(length - 1) == ']') {
            split = value.substring(1, length - 1).split(", ");
        }
        else {
            split = new String[] { value };
        }
        this.value = value;
        try {
            this.encoded = this.encode(objectIdentifier, split);
        }
        catch (IOException cause2) {
            throw new IllegalArgumentException("Incorrect format: value", cause2);
        }
        return;
    }
    
    public PKCS12Attribute(byte[] array) {
        this.hashValue = -1;
        if (array == null) {
            throw new NullPointerException();
        }
        this.encoded = (byte[])array.clone();
        try {
            this.parse(array);
        }
        catch (IOException cause) {
            throw new IllegalArgumentException("Incorrect format: encoded", cause);
        }
        return;
    }
    
    @Override
    public String getName() {
        return this.name;
    }
    
    @Override
    public String getValue() {
        return this.value;
    }
    
    public byte[] getEncoded() {
        return (byte[])this.encoded.clone();
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof PKCS12Attribute && Arrays.equals(this.encoded, ((PKCS12Attribute)o).getEncoded()));
    }
    
    @Override
    public int hashCode() {
        if (this.hashValue == -1) {
            Arrays.hashCode(this.encoded);
        }
        return this.hashValue;
    }
    
    @Override
    public String toString() {
        return this.name + "=" + this.value;
    }
    
    private byte[] encode(ObjectIdentifier objectIdentifier, String[] array) throws IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        derOutputStream.putOID(objectIdentifier);
        DerOutputStream derOutputStream2 = new DerOutputStream();
        for (String input : array) {
            if (PKCS12Attribute.COLON_SEPARATED_HEX_PAIRS.matcher(input).matches()) {
                byte[] original = new BigInteger(input.replace(":", ""), 16).toByteArray();
                if (original[0] == 0) {
                    original = Arrays.copyOfRange(original, 1, original.length);
                }
                derOutputStream2.putOctetString(original);
            }
            else {
                derOutputStream2.putUTF8String(input);
            }
        }
        derOutputStream.write((byte)49, derOutputStream2);
        DerOutputStream o = new DerOutputStream();
        o.write((byte)48, derOutputStream);
        return o.toByteArray();
    }
    
    private void parse(byte[] data) throws IOException {
        DerValue[] sequence = new DerInputStream(data).getSequence(2);
        if (sequence.length != 2) {
            throw new IOException("Invalid length for PKCS12Attribute");
        }
        ObjectIdentifier oid = sequence[0].getOID();
        DerValue[] set = new DerInputStream(sequence[1].toByteArray()).getSet(1);
        String[] a = new String[set.length];
        for (int i = 0; i < set.length; ++i) {
            if (set[i].tag == 4) {
                a[i] = Debug.toString(set[i].getOctetString());
            }
            else {
                String asString;
                if ((asString = set[i].getAsString()) != null) {
                    a[i] = asString;
                }
                else if (set[i].tag == 6) {
                    a[i] = set[i].getOID().toString();
                }
                else if (set[i].tag == 24) {
                    a[i] = set[i].getGeneralizedTime().toString();
                }
                else if (set[i].tag == 23) {
                    a[i] = set[i].getUTCTime().toString();
                }
                else if (set[i].tag == 2) {
                    a[i] = set[i].getBigInteger().toString();
                }
                else if (set[i].tag == 1) {
                    a[i] = String.valueOf(set[i].getBoolean());
                }
                else {
                    a[i] = Debug.toString(set[i].getDataBytes());
                }
            }
        }
        this.name = oid.toString();
        this.value = ((a.length == 1) ? a[0] : Arrays.toString(a));
        return;
    }
    
    static {
        COLON_SEPARATED_HEX_PAIRS = Pattern.compile("^[0-9a-fA-F]{2}(:[0-9a-fA-F]{2})+$");
    }
}
